import unittest
from datetime import datetime

from pyspark.sql import SparkSession
from pyspark.sql.functions import col, format_string
from trend_etl.test_utils import getspark, assert_dataframes_equal, get_df, get_kafka_df
from trend_etl.functions.index import (
    merge_drop_duplicates,
    merge_drop_duplicates_key,
    annotation_matches_text,
    reddit_docs_select_earliest,
    select_earliest_by_key,
    select_recent_by_key,
)
from trend_etl.transform.submissions import (
    from_kafka_get_bronze_submissions_df,
    from_bronze_get_submissions_df_append,
)


class TestFunctions(unittest.TestCase):
    spark: SparkSession

    @classmethod
    def setUpClass(cls) -> None:
        cls.spark = getspark()

    def test_merge_drop_dupes_noop(self) -> None:
        data = [("apple", 1, 42), ("lemon", 1, 69)]
        df = self.__class__.spark.createDataFrame(
            data, schema="fruit string, picked_on int, number_collected int"
        )
        got1 = merge_drop_duplicates(df, "keep-oldest", "fruit", "picked_on")
        got2 = merge_drop_duplicates(df, "keep-newest", "fruit", "picked_on")
        assert_dataframes_equal(got1, df)
        assert_dataframes_equal(got2, df)

    def test_merge_drop_dupes_oldest(self) -> None:
        data = [("apple", 1, 42), ("apple", 2, 1337), ("lemon", 1, 69)]
        df = self.__class__.spark.createDataFrame(
            data, schema="fruit string, picked_on int, number_collected int"
        )
        got = merge_drop_duplicates(df, "keep-oldest", "fruit", "picked_on")
        data = [("apple", 1, 42), ("lemon", 1, 69)]
        want = self.__class__.spark.createDataFrame(
            data, schema="fruit string, picked_on int, number_collected int"
        )
        assert_dataframes_equal(got, want)

    def test_merge_drop_dupes_newest(self) -> None:
        data = [("apple", 1, 42), ("apple", 2, 1337), ("lemon", 1, 69)]
        df = self.__class__.spark.createDataFrame(
            data, schema="fruit string, picked_on int, number_collected int"
        )
        got = merge_drop_duplicates(df, "keep-newest", "fruit", "picked_on")
        data = [("apple", 2, 1337), ("lemon", 1, 69)]
        want = self.__class__.spark.createDataFrame(
            data, schema="fruit string, picked_on int, number_collected int"
        )
        assert_dataframes_equal(got, want)

    def test_merge_drop_dupes_dupes(self) -> None:
        data = [("apple", 1, 42), ("apple", 1, 24), ("lemon", 1, 69)]
        df = self.__class__.spark.createDataFrame(
            data, schema="fruit string, picked_on int, number_collected int"
        )
        got = merge_drop_duplicates(df, "keep-oldest", "fruit", "picked_on")
        self.assertEqual(2, got.count(), "dataframe size different")

    def test_merge_drop_dupes_key(self) -> None:
        data = [
            ("apple", ("red", "Sept"), (None, "👎")),
            ("apple", ("green", "Oct"), (None, "👍")),
            ("apple", ("green", "Oct"), (None, "👍")),
            ("apple", ("blue", "Jun"), (None, "🤨")),
        ]
        df = self.__class__.spark.createDataFrame(
            data,
            schema="fruit string, picked struct<color string, month string>, emoji struct<id string, name string>",
        )
        got = merge_drop_duplicates_key(
            df,
            format_string(
                "%s%s%s", col("fruit"), col("picked.color"), col("emoji.name")
            ),
        )
        self.assertEqual(len(data) - 1, got.count(), "dataframe size different")

    def test_annotation_matches_text(self) -> None:
        docs = get_df(
            self.__class__.spark,
            "doc_uuid string, texts struct<title string, text string>",
            "test_data",
            "docs.jsonl",
        )
        nlp_results = get_df(
            self.__class__.spark,
            "doc_uuid string, annotation string",
            "test_data",
            "historical_nlp_results.jsonl",
        )

        joined = nlp_results.join(docs, "doc_uuid")

        self.assertEqual(joined.count(), nlp_results.count(), "count mismatch")

        failed_matches = [
            row["doc_uuid"]
            for row in joined.where(
                ~annotation_matches_text(col("texts.text"), col("annotation"))
            )
            .selectExpr("doc_uuid")
            .collect()
        ]

        self.assertEqual(failed_matches, ["doc-fail-1"])

    def test_select_earliest_version_reddit_docs(self) -> None:
        source_df = get_kafka_df(self.__class__.spark, "test_data", "submissions.jsonl")
        subreddit_idents = get_df(
            self.__class__.spark,
            "group_id string, id string",
            "test_data",
            "subreddit_idents.jsonl",
        )
        submission_idents = get_df(
            self.__class__.spark,
            "doc_uuid string, id string",
            "test_data",
            "submission_idents.jsonl",
        )
        got_df = from_bronze_get_submissions_df_append(
            from_kafka_get_bronze_submissions_df(source_df, "submissions"),
            subreddit_idents,
            submission_idents,
        )

        got_df = reddit_docs_select_earliest(got_df)

        want_df = got_df.where(
            (col("text") != "[removed]") & (col("text") != "[deleted]")
        )

        assert_dataframes_equal(got_df, want_df)

    def test_select_earliest_by_key(self) -> None:
        data = [
            ("group-1", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-1", ("100", "Alice"), datetime(2023, 8, 1, 10, 0, 0)),
            ("group-2", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-1", ("200", "Bob"), datetime(2024, 8, 1, 10, 0, 0)),
        ]
        input_df = self.__class__.spark.createDataFrame(
            data,
            schema="group_id string, user struct<id string, username string>, collected_on timestamp",
        )

        got_df = select_earliest_by_key(input_df, ["group_id", "user.id"])

        data = [
            ("group-1", ("100", "Alice"), datetime(2023, 8, 1, 10, 0, 0)),
            ("group-1", ("200", "Bob"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-2", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
        ]
        want_df = self.__class__.spark.createDataFrame(
            data,
            schema="group_id string, user struct<id string, username string>, collected_on timestamp",
        )

        assert_dataframes_equal(got_df, want_df)

    def test_select_recent_by_key(self) -> None:
        data = [
            ("group-1", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-1", ("100", "Alice"), datetime(2023, 8, 1, 10, 0, 0)),
            ("group-2", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-1", ("200", "Bob"), datetime(2024, 8, 1, 10, 0, 0)),
        ]
        input_df = self.__class__.spark.createDataFrame(
            data,
            schema="group_id string, user struct<id string, username string>, collected_on timestamp",
        )

        got_df = select_recent_by_key(input_df, ["group_id", "user.id"])

        data = [
            ("group-1", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-1", ("200", "Bob"), datetime(2024, 8, 1, 10, 0, 0)),
            ("group-2", ("100", "Alice"), datetime(2024, 8, 1, 10, 0, 0)),
        ]
        want_df = self.__class__.spark.createDataFrame(
            data,
            schema="group_id string, user struct<id string, username string>, collected_on timestamp",
        )

        assert_dataframes_equal(got_df, want_df)
