import argparse
from typing import cast

from delta.tables import DeltaTable
from pyspark.sql import SparkSession

from trend_etl.functions.index import full_table_name, merge_recent_collected_by_key
from trend_etl.transform.channels import from_bronze_get_channels_df
import trend_etl.schema.args as arguments
import trend_etl.schema.fields as fields

program = "load_channels_append"


def main() -> None:
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{program}.")

    parser.add_argument(
        "--bronze-catalog",
        dest="bronze_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--target-catalog",
        dest="target_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--checkpoints-path-prefix",
        dest="checkpoints_path_prefix",
        required=True,
        help="The path to the folder (in S3) to store the checkpoints for this table",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(program).getOrCreate())

    kafka_channels_table = full_table_name(
        args.bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_channels],
    )
    streaming_kafka_channels = spark.readStream.format("delta").table(
        kafka_channels_table
    )

    guild_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_guild_idents],
    )
    guild_idents = spark.read.format("delta").table(guild_idents_table)

    channel_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_channel_idents],
    )
    channel_idents = spark.read.format("delta").table(channel_idents_table)

    channel_append_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_discord_channels_append],
    )

    prepared = from_bronze_get_channels_df(
        streaming_kafka_channels, guild_idents, channel_idents
    )

    checkpoint_loc = f"{args.checkpoints_path_prefix}/discord_channels_append"
    delta_table = DeltaTable.forName(spark, channel_append_table)

    (
        prepared.writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: merge_recent_collected_by_key(
                delta_table, batch_df, [fields.idents_doc_id_field]
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


if __name__ == "__main__":
    main()
