import argparse
from typing import cast

from delta.tables import DeltaTable
from pyspark.sql import SparkSession

from trend_etl.functions.index import full_table_name, merge_earliest_collected_by_key
from trend_etl.transform.members import from_bronze_get_members_df
import trend_etl.schema.args as arguments
import trend_etl.schema.fields as fields

program = "load_members_append"


def main() -> None:
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{program}.")

    parser.add_argument(
        "--bronze-catalog",
        dest="bronze_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--target-catalog",
        dest="target_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--checkpoints-path-prefix",
        dest="checkpoints_path_prefix",
        required=True,
        help="The path to the folder (in S3) to store the checkpoints for this table",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(program).getOrCreate())

    kafka_members_table = full_table_name(
        args.bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_members],
    )
    streaming_kafka_members = spark.readStream.format("delta").table(
        kafka_members_table
    )

    guild_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_guild_idents],
    )
    guild_idents = spark.read.format("delta").table(guild_idents_table)

    member_append_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_discord_members_append],
    )

    prepared = from_bronze_get_members_df(streaming_kafka_members, guild_idents)

    checkpoint_loc = f"{args.checkpoints_path_prefix}/discord_members_append"
    delta_table = DeltaTable.forName(spark, member_append_table)

    (
        prepared.writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: merge_earliest_collected_by_key(
                delta_table, batch_df, [fields.idents_group_id_field, "user.id"]
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


if __name__ == "__main__":
    main()
