import argparse
from typing import cast

from delta.tables import DeltaTable
from pyspark.sql import SparkSession

from trend_etl.functions.index import (
    full_table_name,
    merge_earliest_collected_reddit_docs,
    get_silver_table,
)
from trend_etl.transform.replies import from_bronze_get_replies_df_append
import trend_etl.schema.args as arguments

program = "load_replies_append"


def main() -> None:
    parser = argparse.ArgumentParser(description=f"Arguments for Trend ETL/{program}.")

    parser.add_argument(
        "--bronze-catalog",
        dest="bronze_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--target-catalog",
        dest="target_catalog",
        required=True,
        choices=arguments.catalogs,
        help="The unit catalog to use",
    )

    parser.add_argument(
        "--checkpoints-path-prefix",
        dest="checkpoints_path_prefix",
        required=True,
        help="The path to the folder (in S3) to store the checkpoints for this table",
    )

    args = parser.parse_args()

    spark = cast(SparkSession, SparkSession.builder.appName(program).getOrCreate())

    kafka_replies_table = full_table_name(
        args.bronze_catalog,
        arguments.schemas[arguments.schemas_bronze],
        arguments.tables[arguments.tables_replies],
    )
    streaming_kafka_replies = spark.readStream.format("delta").table(
        kafka_replies_table
    )

    subreddit_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_subreddit_idents],
    )
    subreddit_idents = spark.read.format("delta").table(subreddit_idents_table)

    submission_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_submission_idents],
    )
    submission_idents = spark.read.format("delta").table(submission_idents_table)

    reply_idents_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        arguments.tables[arguments.tables_reply_idents],
    )
    reply_idents = spark.read.format("delta").table(reply_idents_table)

    reddit_replies_append_table = full_table_name(
        args.target_catalog,
        arguments.schemas[arguments.schemas_silver],
        get_silver_table(args.target_catalog, arguments.tables_reddit_replies_append),
    )

    prepared = from_bronze_get_replies_df_append(
        streaming_kafka_replies,
        subreddit_idents,
        submission_idents,
        reply_idents,
    )

    checkpoint_loc = f"{args.checkpoints_path_prefix}/reddit_replies_append"
    delta_table = DeltaTable.forName(spark, reddit_replies_append_table)

    (
        prepared.writeStream.format("delta")
        .foreachBatch(
            lambda batch_df, batch_id: merge_earliest_collected_reddit_docs(
                delta_table, batch_df
            )
        )
        .option("checkpointLocation", checkpoint_loc)
        .outputMode("append")
        .trigger(availableNow=True)
        .start()
    )


if __name__ == "__main__":
    main()
